<?php

namespace Drupal\membership_commerce\EventSubscriber;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\commerce_order\OrderAssignmentInterface;
use Drupal\commerce_order\Event\OrderAssignEvent;
use Drupal\commerce_order\Event\OrderEvents;
use Drupal\group\Entity\Group;
use Drupal\membership\Event\MembershipEvent;
use Drupal\membership\Event\MembershipEvents;
use Drupal\state_machine\Event\WorkflowTransitionEvent;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * membership_commerce event subscriber.
 */
class MembershipCommerceSubscriber implements EventSubscriberInterface {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The commerce_order.order_assignment service.
   *
   * @var \Drupal\commerce_order\OrderAssignmentInterface
   */
  protected $orderAssignment;

  /**
   * The messenger.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * Constructs event subscriber.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\commerce_order\OrderAssignmentInterface $order_assignment
   *   The commerce_order.order_assignment service.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, OrderAssignmentInterface $order_assignment, MessengerInterface $messenger) {
    $this->entityTypeManager = $entity_type_manager;
    $this->orderAssignment = $order_assignment;
    $this->messenger = $messenger;
  }

  /**
   * Handle membership activation, expiration.
   *
   * @param \Drupal\membership\Event\MembershipEvent $event
   *
   */
  public function HandleMembershipChange(MembershipEvent $event) {
    $membership = $event->getMembership();

  }

  /**
   * When membership is activated, publish corresponding group. Create new group
   * if it doesn't already exist.
   *
   * @param \Drupal\state_machine\Event\WorkflowTransitionEvent $event
   */
  public function ActivateMembership(WorkflowTransitionEvent $event) {
    /** @var \Drupal\membership_term\Entity\MembershipTermInterface $membership_term */
    $membership_term = $event->getEntity();
    $membership = $membership_term->getMembership();
    if (!$membership->field_group->isEmpty()) {
      $group = $membership->field_group->entity;
      $group->status->value = 1;
      $group->field_user_count->value = $membership->field_user_count->value;
      $group->save();
    }
    $membership->save();
  }

  /**
   * When membership goes inactive, unpublish corresponding group.
   *
   * @param \Drupal\state_machine\Event\WorkflowTransitionEvent $event
   */
  public function DeactivateMembership(WorkflowTransitionEvent $event) {
    $membership_term = $event->getEntity();
    $membership = $membership_term->getMembership();
    $group = $membership->field_group->entity;
    $group->status->value = FALSE;
    $group->save();

  }
  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    return [
      MembershipEvents::STATE_CHANGE => ['HandleMembershipChange'],
      'membership_term.activate.post_transition' => ['ActivateMembership'],
      'membership_term.expire.post_transition' => ['DeactivateMembership'],
      OrderEvents::ORDER_ASSIGN => ['orderAssign'],
    ];
  }

  /**
   * Updates the subscription customer when an order is reassigned.
   *
   * @param Drupal\commerce_order\Event\OrderAssignEvent $event
   *   The order assignment event.
   */
  public function orderAssign(OrderAssignEvent $event) {
    $order = $event->getOrder();
    $customer = $event->getCustomer();
    $subscription_storage = $this->entityTypeManager->getStorage('commerce_subscription');
    $query = $subscription_storage->getQuery();
    $query
      ->condition('initial_order', $order->id());
    $result = $query->execute();
    if ($result) {
      /** @var \Drupal\commerce_recurring\Entity\SubscriptionInterface $subscription */
      foreach ($subscription_storage->loadMultiple($result) as $subscription) {
        // Ensure the subscription customer matches the order customer.
        if ($subscription->getCustomerId() != $customer->id()) {
          $subscription->setCustomer($customer);
          $subscription->save();
        }

        // Ensure all subscription orders are also are re-assigned.
        /** @var \Drupal\commerce_order\Entity\OrderInterface $subscription_order */
        foreach ($subscription->getOrders() as $subscription_order) {
          if ($subscription_order->getCustomerId() != $customer->id()) {
            $this->orderAssignment->assign($subscription_order, $customer);
          }
        }
      }
    }
  }
}
